import Qt 4.7
import QtQuick 1.0

Item {
    id: root
    height: 80
    property bool alsEnabled: false
    property string color_level: "/desktop/meego/status_menu/brightness-qml/color_level"
    property string color_level_border: "/desktop/meego/status_menu/brightness-qml/color_level_border"
    property string color_system: "/desktop/meego/status_menu/brightness-qml/color_system"
    property string color_system_border: "/desktop/meego/status_menu/brightness-qml/color_system_border"
    property string color_background: "/desktop/meego/status_menu/brightness-qml/color_background"
    property string color_background_border: "/desktop/meego/status_menu/brightness-qml/color_background_border"
    property string gconf_als: "/system/osso/dsm/display/als_enabled"
    property string brightness_level: "/system/osso/dsm/display/display_brightness"
    property string sys_brightness: "/sys/devices/omapdss/display0/backlight/display0/brightness"

    property int brightnessStep: 0

    property bool isOriginTrusted: false

    property int _ZONE1: 468 / 4;
    property int _ZONE2: _ZONE1 + _ZONE1; //(BAR_WIDTH / 4) * 2
    property int _ZONE3: _ZONE1 + _ZONE2; //(BAR_WIDTH / 4) * 3

    function stepPos(pos, isMove) {
        if (pos > _ZONE3)
            return 5
        else if (pos > _ZONE2)
            return 4
        else if (pos > _ZONE1)
            return 3
        else if (isMove && pos > 58)
            return 2
        else if (isMove)
            return 1
        else if (!isMove && brightnessStep == 1)
            return 2
        else if (!isMove)
            return 1
    }

    function percentPos(pos) {
        var val = pos * 255 / 468
        return parseInt(val)
    }

    function posPercent(percent) {
        var w = 468 * parseInt(percent) / 255;
        return w
    }

    function posStep(step) {
        var w = 468 * (step-1) / 4
        return w
    }

    function displayEvent(isEnter) {
        if (isEnter) {
            if (root.alsEnabled) {
                brightnessStep = gconf.get(brightness_level)
                rectOne.width = posStep(brightnessStep)
            }
            else
                getSystemBrightness()
        }
    }

    function getSystemBrightness() {
        var ret1 = dbus.sendCommand("READ", sys_brightness)
        rectOne.width = posPercent(JSON.parse(ret1).value)
    }

    function getColors() {
        if (root.alsEnabled) {
            rectOne.color = gconf.get(root.color_level, "#FF0000")
            rectOne.border.color = gconf.get(root.color_level_border, "#FF0000")
        }
        else {
            rectOne.color = gconf.get(root.color_system, "#00FF00")
            rectOne.border.color = gconf.get(root.color_system_border, "#00FF00")
        }
        rectBg.color = gconf.get(root.color_background, "#323232")
        rectBg.border.color = gconf.get(root.color_background_border, "#323232")
    }

    Component.onCompleted: {
        root.alsEnabled = gconf.get(gconf_als)
        gconf.addWatcher(gconf_als)
        gconf.addWatcher(brightness_level)
        gconf.addWatcher(color_level)
        gconf.addWatcher(color_system)
        gconf.addWatcher(color_level_border)
        gconf.addWatcher(color_system_border)
        gconf.addWatcher(color_background)
        gconf.addWatcher(color_background_border)
        root.brightnessStep = gconf.get(brightness_level)
        root.getColors()
        var origin = dbus.sendCommand("ORIGIN", "")
        isOriginTrusted = JSON.parse(origin).value
        if (isOriginTrusted) {
            if (!root.alsEnabled) {
                dbus.sendCommand("brightness_watch", "START")
                getSystemBrightness()
            }
            else {
                dbus.sendCommand("brightness_watch", "STOP")
                rectOne.width = posStep(brightnessStep)
            }
        }
        else
        {
            testText.text = "Server lack of credentials."
            testText.color = "red"
        }
    }

    Connections {
        target: gconf
        onValueChanged: {
            switch (key) {
                case gconf_als: {
                    root.alsEnabled = value
                    if (!root.alsEnabled) {
                        dbus.sendCommand("brightness_watch", "START")
                        getSystemBrightness()
                    }
                    else {
                        dbus.sendCommand("brightness_watch", "STOP")
                        brightnessStep = gconf.get(brightness_level)
                        rectOne.width = posStep(brightnessStep)
                    }
                    root.getColors()
                    break
                }
                case brightness_level: {
                    brightnessStep = value
                    if (root.alsEnabled) {
                        rectOne.width = posStep(brightnessStep)
                    }
                    break
                }
                case color_system: {
                    if (!root.alsEnabled)
                        rectOne.color = value
                    break
                }
                case color_level: {
                    if (root.alsEnabled)
                        rectOne.color = value
                    break
                }
                case color_system_border: {
                    if (!root.alsEnabled)
                        rectOne.border.color = value
                    break
                }
                case color_level_border: {
                    if (root.alsEnabled)
                        rectOne.border.color = value
                    break
                }
                case color_background: {
                    rectBg.color = value
                    break
                }
                case color_background_border: {
                    rectBg.border.color = value
                    break
                }
            }
        }
    }

    Text {
        id: testText
        anchors.top: parent.top
        anchors.topMargin: -4
        anchors.left: parent.left
        anchors.leftMargin: 6
        font.pixelSize: 22
        color: "white"
        text: qsTrId("qtn_sett_main_brightness")
    }

    Rectangle {
        id: rectBg
        width: 468
        height: 24
        anchors.top: testText.bottom
        anchors.topMargin: 4
        anchors.left: parent.left
        anchors.leftMargin: 6
        color: "#323232"
        radius: 2
        border.width: 1
        border.color: "#323232"
        smooth: true
    }

    Rectangle {
        id: rectOne
        width: 0
        height: 24
        anchors.top: rectBg.top
        anchors.left: rectBg.left
        color: root.alsEnabled ? "red" : "green"
        radius: 2
        border.width: 1
        border.color: root.alsEnabled ? "red" : "green"
        smooth: true
    }

    Timer {
        id: longTimer
        interval: 800
        repeat: false
        onTriggered: {
            gconf.set(gconf_als, !alsEnabled)
            root.getColors()
        }
    }

    MouseArea {
        id: rectArea
        anchors.fill: rectBg
        property int startX: 0
        property int startY: 0
        onPositionChanged: {
            var distance = Math.sqrt(Math.pow(startX - mouse.x, 2) + Math.pow(startY - mouse.y, 2))
            if (distance > 15)
                longTimer.running = false
            if (root.alsEnabled) {
                brightnessStep = stepPos(mouse.x, true)
                gconf.set(brightness_level, brightnessStep)
            }
            else {
                var bright = percentPos(mouse.x)
                if (bright < 1)
                    bright = 1
                else if (bright > 255)
                    bright = 255
                rectOne.width = posPercent(bright)
                dbus.sendCommand("brightness", bright)
            }
        }
        onPressed: {
            startX = mouse.x
            startY = mouse.y
            longTimer.running = true
            if (root.alsEnabled) {
                brightnessStep = stepPos(mouse.x, false)
                gconf.set(brightness_level, brightnessStep)
            }
            else {
                var bright = percentPos(mouse.x)
                if (bright < 1)
                    bright = 1
                else if (bright > 255)
                    bright = 255
                rectOne.width = posPercent(bright)
                dbus.sendCommand("brightness", bright)
            }
        }
        onReleased: {
            longTimer.running = false
        }
    }
}
